<?php


namespace App\Mobile\Synchronization\Controllers;


use App\Domain\Logs\Model\ErrorLog;
use App\Domain\Logs\Service\SaveError;
use App\Http\Controller;
use App\Mobile\Synchronization\Managers\SyncManager;
use App\Mobile\Synchronization\Requests\SyncRouteRequest;
use App\Mobile\Synchronization\Requests\UploadSyncRouteRequest;
use App\Mobile\Synchronization\Resources\SyncClientResource;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Request;
use Schaffen\Financial\BillingRoute\Repositories\RouteRepository;
use Schaffen\Financial\FinancialMovement\Models\FinancialMovement;
use Schaffen\Financial\FinancialMovement\Models\ImportedFinancialMovement;
use Schaffen\Financial\FinancialMovement\Repositories\FinancialMovementRepository;
use Schaffen\Financial\FinancialMovement\Services\CalculationHandle;

class SyncChargeController extends Controller
{
    private FinancialMovementRepository $financialMovementRepository;
    private RouteRepository $routeRepository;
    private SyncManager $manager;

    public function __construct(
        FinancialMovementRepository $financialMovementRepository,
        RouteRepository $routeRepository,
        SyncManager $manager
    )
    {
        $this->financialMovementRepository = $financialMovementRepository;
        $this->routeRepository = $routeRepository;
        $this->manager = $manager;
    }

    public function sync(SyncRouteRequest $request)
    {
        $input = $request->validated();
        DB::beginTransaction();
        $route = $this->routeRepository->find($input['route_id'], ['clients']);
        $vencimento = now()->endOfMonth()->endOfDay();
        $mvfs = $this->financialMovementRepository->list([
            "mvf_status" => FinancialMovement::STATUS_ABERTO,
            "pes_codigo_in" => $route->clients->pluck('pes_codigo'),
            "par_vencimento_end" => $vencimento,

        ],["installment" => function($query) use($vencimento) {
            $query->where('par_data_vencimento_atual', '<=', $vencimento);
            return $query;
        }]);

        $mvfs->each(function ($mvf){
            $mvf->mvf_status = FinancialMovement::STATUS_BLOQUEADO_COBRANCA;
            $mvf->save();
            $mvf->installment->each(function (&$installment) use($mvf){
                $installment->total = CalculationHandle::installmentTotal($installment, $mvf);
                ImportedFinancialMovement::create([
                    "mfim_fk_mvf_codigo" => $mvf->mvf_codigo,
                    "mfim_fk_par_codigo" => $installment->par_codigo,
                    "mfim_status" => ImportedFinancialMovement::STATUS_IMPORTADO_MOVEL
                ]);
            });
        });
        $clients = $route->clients->map(function ($client) use($mvfs){
            $client->mvfs = $mvfs->filter(function ($mvf, $key) use ($client){
                return $mvf->mvf_fk_pes_codigo == $client->pes_codigo;
            });
            return $client;
        })->filter(function ($client, $key){
            return count($client->mvfs) > 0;
        });
        DB::commit();
        //dd($clients->toArray());
        return SyncClientResource::collection($clients);
    }

    public function uploadSync(UploadSyncRouteRequest $request)
    {
        try {
            $this->manager->uploadSyncRoute($request->validated());
        }catch (\Exception $e){
            SaveError::generateLog(ErrorLog::MOVEL_SYNC_ROUTE, json_encode($request->all()), $e->getMessage());
            throw $e;
        }
        return response('Ok', 201);
    }
}
